@@ -11,8 +11,15 @@ class ServerVariable(BaseModel):
     enum: Optional[List[str]] = None
     description: Optional[str] = None
 
+    class Config:
+        extra = "allow"
+
 
 class Server(BaseModel):
+    """https://spec.openapis.org/oas/v3.0.3#server-object"""
     url: str
     description: Optional[str] = None
     variables: Optional[Dict[str, ServerVariable]] = None
+
+    class Config:
+        extra = "allow"
