@@ -1,6 +1,9 @@
 # -*- coding: utf-8 -*-
 # @Author  : llc
 # @Time    : 2022/8/30 9:40
+import functools
+import inspect
+import sys
 from abc import ABC
 from functools import wraps
 from typing import Callable, List, Optional, Dict, Type, Any, Tuple
@@ -9,31 +12,43 @@ from flask.scaffold import Scaffold
 from flask.wrappers import Response
 from pydantic import BaseModel
 
-from .do_wrapper import _do_wrapper
 from .http import HTTPMethod
 from .models import ExternalDocumentation
 from .models.common import ExtraRequestBody
 from .models.server import Server
 from .models.tag import Tag
+from .request import _do_request
+
+if sys.version_info >= (3, 8):
+    iscoroutinefunction = inspect.iscoroutinefunction
+else:
+    def iscoroutinefunction(func: Any) -> bool:
+        while inspect.ismethod(func):
+            func = func.__func__
+
+        while isinstance(func, functools.partial):
+            func = func.func
+
+        return inspect.iscoroutinefunction(func)
 
 
-class _Scaffold(Scaffold, ABC):
+class APIScaffold(Scaffold, ABC):
     def _do_decorator(
             self,
             rule: str,
             func: Callable,
             *,
-            tags: List[Tag] = None,
+            tags: Optional[List[Tag]] = None,
             summary: Optional[str] = None,
             description: Optional[str] = None,
             external_docs: Optional[ExternalDocumentation] = None,
             operation_id: Optional[str] = None,
             extra_form: Optional[ExtraRequestBody] = None,
             extra_body: Optional[ExtraRequestBody] = None,
-            responses: Dict[str, Type[BaseModel]] = None,
-            extra_responses: Dict[str, dict] = None,
+            responses: Optional[Dict[str, Optional[Type[BaseModel]]]] = None,
+            extra_responses: Optional[Dict[str, dict]] = None,
             deprecated: Optional[bool] = None,
-            security: List[Dict[str, List[Any]]] = None,
+            security: Optional[List[Dict[str, List[Any]]]] = None,
             servers: Optional[List[Server]] = None,
             doc_ui: bool = True,
             method: str = HTTPMethod.GET
@@ -43,6 +58,57 @@ class _Scaffold(Scaffold, ABC):
     def register_api(self, api) -> None:
         raise NotImplementedError
 
+    @staticmethod
+    def create_view_func(func, header, cookie, path, query, form, body, view_class=None):
+        is_coroutine_function = iscoroutinefunction(func)
+        if is_coroutine_function:
+            @wraps(func)
+            async def view_func(**kwargs) -> Response:
+                result = _do_request(
+                    header=header,
+                    cookie=cookie,
+                    path=path,
+                    query=query,
+                    form=form,
+                    body=body,
+                    **kwargs
+                )
+                if isinstance(result, Response):
+                    # 422
+                    return result
+                # handle async request
+                if view_class:
+                    response = await func(view_class, **result)
+                else:
+                    response = await func(**result)
+                return response
+        else:
+            @wraps(func)
+            def view_func(**kwargs) -> Response:
+                result = _do_request(
+                    header=header,
+                    cookie=cookie,
+                    path=path,
+                    query=query,
+                    form=form,
+                    body=body,
+                    **kwargs
+                )
+                if isinstance(result, Response):
+                    # 422
+                    return result
+                # handle request
+                if view_class:
+                    response = func(view_class, **result)
+                else:
+                    response = func(**result)
+                return response
+
+        if not hasattr(func, "view"):
+            func.view = view_func
+
+        return func.view
+
     def get(
             self,
             rule: str,
@@ -104,24 +170,9 @@ class _Scaffold(Scaffold, ABC):
                     method=HTTPMethod.GET
                 )
 
-            @wraps(func)
-            def wrapper(**kwargs) -> Response:
-                resp = _do_wrapper(
-                    func,
-                    header=header,
-                    cookie=cookie,
-                    path=path,
-                    query=query,
-                    form=form,
-                    body=body,
-                    **kwargs
-                )
-                return resp
-
+            view_func = self.create_view_func(func, header, cookie, path, query, form, body)
             options.update({"methods": [HTTPMethod.GET]})
-            if not hasattr(func, "wrapper"):
-                func.wrapper = wrapper
-            self.add_url_rule(rule, view_func=func.wrapper, **options)
+            self.add_url_rule(rule, view_func=view_func, **options)
 
             return func
 
@@ -188,24 +239,9 @@ class _Scaffold(Scaffold, ABC):
                     method=HTTPMethod.POST
                 )
 
-            @wraps(func)
-            def wrapper(**kwargs) -> Response:
-                resp = _do_wrapper(
-                    func,
-                    header=header,
-                    cookie=cookie,
-                    path=path,
-                    query=query,
-                    form=form,
-                    body=body,
-                    **kwargs
-                )
-                return resp
-
+            view_func = self.create_view_func(func, header, cookie, path, query, form, body)
             options.update({"methods": [HTTPMethod.POST]})
-            if not hasattr(func, "wrapper"):
-                func.wrapper = wrapper
-            self.add_url_rule(rule, view_func=func.wrapper, **options)
+            self.add_url_rule(rule, view_func=view_func, **options)
 
             return func
 
@@ -272,24 +308,9 @@ class _Scaffold(Scaffold, ABC):
                     method=HTTPMethod.PUT
                 )
 
-            @wraps(func)
-            def wrapper(**kwargs) -> Response:
-                resp = _do_wrapper(
-                    func,
-                    header=header,
-                    cookie=cookie,
-                    path=path,
-                    query=query,
-                    form=form,
-                    body=body,
-                    **kwargs
-                )
-                return resp
-
+            view_func = self.create_view_func(func, header, cookie, path, query, form, body)
             options.update({"methods": [HTTPMethod.PUT]})
-            if not hasattr(func, "wrapper"):
-                func.wrapper = wrapper
-            self.add_url_rule(rule, view_func=func.wrapper, **options)
+            self.add_url_rule(rule, view_func=view_func, **options)
 
             return func
 
@@ -356,24 +377,9 @@ class _Scaffold(Scaffold, ABC):
                     method=HTTPMethod.DELETE
                 )
 
-            @wraps(func)
-            def wrapper(**kwargs) -> Response:
-                resp = _do_wrapper(
-                    func,
-                    header=header,
-                    cookie=cookie,
-                    path=path,
-                    query=query,
-                    form=form,
-                    body=body,
-                    **kwargs
-                )
-                return resp
-
+            view_func = self.create_view_func(func, header, cookie, path, query, form, body)
             options.update({"methods": [HTTPMethod.DELETE]})
-            if not hasattr(func, "wrapper"):
-                func.wrapper = wrapper
-            self.add_url_rule(rule, view_func=func.wrapper, **options)
+            self.add_url_rule(rule, view_func=view_func, **options)
 
             return func
 
@@ -440,24 +446,9 @@ class _Scaffold(Scaffold, ABC):
                     method=HTTPMethod.PATCH
                 )
 
-            @wraps(func)
-            def wrapper(**kwargs) -> Response:
-                resp = _do_wrapper(
-                    func,
-                    header=header,
-                    cookie=cookie,
-                    path=path,
-                    query=query,
-                    form=form,
-                    body=body,
-                    **kwargs
-                )
-                return resp
-
+            view_func = self.create_view_func(func, header, cookie, path, query, form, body)
             options.update({"methods": [HTTPMethod.PATCH]})
-            if not hasattr(func, "wrapper"):
-                func.wrapper = wrapper
-            self.add_url_rule(rule, view_func=func.wrapper, **options)
+            self.add_url_rule(rule, view_func=view_func, **options)
 
             return func
 
